﻿module DUtil.CStringBuilder;
import std.algorithm;

class CStringBuilder
{
	private char[] mData;
	private int mSize = 0;
	private int mLength = 0;

	public int GetLength()
	{
		return mLength;
	}



	public string GetString()
	{
		return cast(string)mData[0 .. mSize];
	}



	public void AppendChar(int aChar)
	in
	{
		assert(aChar >= 0 && aChar < 0x110000 && !(aChar >= 0xd800 && aChar < 0xe000));
	}
	body
	{
		if (aChar < 0x80)
		{
			Allocate(mSize + 1);
			mData[mSize++] = cast(char)aChar;
		}
		else if (aChar >= 0x80 && aChar < 0x800)
		{
			Allocate(mSize + 2);
			mData[mSize++] = 0xc0 | ((aChar >> 6) & 0x1f);
			mData[mSize++] = 0x80 | (aChar & 0x3f);
		}
		else if (aChar >= 0x800 && aChar < 0x10000)
		{
			Allocate(mSize + 3);
			mData[mSize++] = 0xe0 | ((aChar >> 12) & 0xf);
			mData[mSize++] = 0x80 | ((aChar >> 6) & 0x3f);
			mData[mSize++] = 0x80 | (aChar & 0x3f);
		}
		else
		{
			Allocate(mSize + 4);
			mData[mSize++] = 0xf0 | ((aChar >> 18) & 0x7);
			mData[mSize++] = 0x80 | ((aChar >> 12) & 0x3f);
			mData[mSize++] = 0x80 | ((aChar >> 6) & 0x3f);
			mData[mSize++] = 0x80 | (aChar & 0x3f);
		}
		mLength++;
	}



	private void Allocate(int aCapacity)
	{
		if (mData.length >= aCapacity)
			return;
		mData.length = max(max(mData.length + mData.length, aCapacity), 8);
	}



	unittest
	{
		auto s = new CStringBuilder();
		s.AppendChar(0x2014);
		assert(s.GetString() == [ 0xe2, 0x80, 0x94 ]);
		assert(s.GetLength() == 1);
	}
}