﻿module Obelisk.IObelisk;
import Obelisk.CGrid, Obelisk.CColour, Obelisk.CTexture, Obelisk.CVector, Obelisk.EKey;

enum ECoordinate { X, NegativeX, Y, NegativeY, Z, NegativeZ }

interface IObelisk
{
	void SetDirectory(string aDirectory);
	void SetCaption(string aCaption);
	void SetSize(int aXSize, int aYSize);
	void SetShaders(bool aUseShaders);

	void Quit();

	void StartTimer();
	void StopTimer();

	uint CreateTimer();
	uint Duration(uint aTimer);

	bool MouseButtonDown(bool aRight);
	bool MouseButtonClicked(bool aRight);
	bool MouseButtonReleased(bool aRight);
	void GetCursorPos(out int aCursorX, out int aCursorY);

	bool IsKeyPressed(EKey aKey);
	bool IsKeyDown(EKey aKey);

	// graphics functions
	void LoadTexture(string aName);
	CTexture CreateTexture(string aName, uint[] aData, int aXSize, int aYSize);
	CTexture GetTexture(string aName);
	void LoadShader(string aName);
	void SetLinearFiltering(bool aLinearFiltering);
	void SetSpriteAlign(bool aSpriteAlign);

	void BeginFrame();
	void EndFrame();
	void SetBackgroundColour(CColour aColour);
	void SetColour(CColour aColour);
	void SetAlpha(float aAlpha);

	void ProjectionPerspective(float aNearWidth, float aNearDepth, float aFarDepth,
			ECoordinate aHorizontalCoord, ECoordinate aVerticalCoord, ECoordinate aDepthCoord);
	void ProjectionUI();

	void DrawSprite(CTexture aTexture, int aX, int aY, int aXSize = -1, int aYSize = -1,
			float aXAlign = 0, float aYAlign = 0);

	void Tri(CVector aV0, CVector aV1, CVector aV2);
	void TriColour(CVector aV0, CVector aV1, CVector aV2, CColour aColour);
	void TriColour(CVector aV0, CColour aC0, CVector aV1, CColour aC1, CVector aV2,
			CColour aC2);
	void TriTexture(CVector aV0, float aS0, float aT0, CVector aV1, float aS1, float aT1,
			CVector aV2, float aS2, float aT2, CTexture aTexture);
	void Quad(CVector aV0, CVector aV1, CVector aV2, CVector aV3);
	void QuadColour(CVector aV0, CVector aV1, CVector aV2, CVector aV3, CColour aColour);
	void QuadColour(CVector aV0, CColour aC0, CVector aV1, CColour aC1, CVector aV2,
			CColour aC2, CVector aV3, CColour aC3);
	void QuadTexture(CVector aV0, float aS0, float aT0, CVector aV1, float aS1, float aT1,
			CVector aV2, float aS2, float aT2, CVector aV3, float aS3, float aT3,
			CTexture aTexture);
	void QuadShader(CVector aV0, float aS0, float aT0, CVector aV1, float aS1, float aT1,
			CVector aV2, float aS2, float aT2, CVector aV3, float aS3, float aT3,
			string aShaderName);
	void ShaderUniform(string aShaderName, string aUniform, float aValue);

	// font/text functions
	void LoadFont(string aName);
	void TextBegin(int aOriginX, int aOriginY, float aAlign, int aLineSpacing = 0);
	void TextEnd();
	//CTexture TextEndTexture(string aName);
	void TextSetFont(string aFontName, int aSize);
	void TextWrite(string aText);

	// audio functions
	void LoadSound(string aName);
	void Sound(string aName);

	// UI functions
	void SetMouseCursorSprite(string aTextureName, int aHotspotX, int aHotspotY);
	void FormBegin();
	int FormButton(int aX, int aY, int aXSize, int aYSize, float aXAlign, string aText);
	bool FormButtonClicked(int aButton);
	void RenderUI();

	CGrid LoadGrid(string aName, int[int] aColourMap);
}